package labone;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

/**
 * Created by IntelliJ IDEA.
 * User: zaic
 * Date: 3/21/12
 * Time: 6:55 PM
 */
public class DynamicImageDrawer {
    PlotPanel panel;
    BufferedImage image;
    boolean blend = false;
    boolean filter = false;
    
    Triangle triangles[];

    DynamicImageDrawer(PlotPanel panel, BufferedImage image, boolean blend, boolean filter) {
        this.panel = panel;
        this.image = image;
        this.blend = blend;
        this.filter = filter;
        
        triangles = new Triangle[32];
        for(int i = 0; i < 4; i++)
            for(int j = 0; j < 4; j++)
                for(int k = 0; k < 2; k++) {
                    triangles[i * 8 + j * 2 + k] = new Triangle(0.25 * i, 0.25 * j);
                }
    }

    void setBlend(boolean value) {
        blend = value;
    }

    void setFilter(boolean value) {
        filter = value;
    }

    /*
     *  rotate
     */
    
    
    Point2D.Double rotate(Point2D.Double p, double angle) {
        double s = Math.sin(angle);
        double c = Math.cos(angle);
        return new Point2D.Double(p.getX() * c - p.getY() * s, p.getX() * s + p.getY() * c);
    }
    
    /*
     *  chessboard
     */
    
    void drawChessboard(int field[][]) {
        boolean startfill = true;
        for(int i = 0; i < field.length; i += StaticImageDrawer.chess_cell_size) {
            boolean fill = startfill;
            startfill = !startfill;
            for(int j = 0; j < field[i].length; j += StaticImageDrawer.chess_cell_size) {
                if(fill)
                    for(int k = 0; k < StaticImageDrawer.chess_cell_size; k++)
                        for(int w = 0; w < StaticImageDrawer.chess_cell_size; w++)
                            field[i + k][j + w] = Color.lightGray.getRGB();
                fill = !fill;
            }
                
        }
    }
    
    /*
     *  triangle
     */
    
    int[][] getTrianglePoints(int triangle_size, int triangle_shift, int start_orientation, double angle, Point2D.Double corners[]) {
        corners[0] = new Point2D.Double(0, triangle_size);
        corners[1] = new Point2D.Double(triangle_size, 0);
        corners[2] = new Point2D.Double(start_orientation == 0 ? 0 : triangle_size, start_orientation == 0 ? 0 : triangle_size);
        for(int i = 0; i < 3; i++) {
            corners[i] = rotate(corners[i], angle);
            //System.err.println("x=" + corners[i].getX() + ", y=" + corners[i].getY());
        }

        int field[][] = new int[3 * triangle_size][3 * triangle_size];
        for(int i = 0; i < field.length; i++) for(int j = 0; j < field[0].length; j++) field[i][j] = -1;
        for(int i = 0; i < 3; i++)
            GraphicsPrimitives.drawLine(field,
                    (int)corners[i].getX() + triangle_shift, (int)corners[i].getY() + triangle_shift,
                    (int)corners[(i + 1) % 3].getX() + triangle_shift, (int)corners[(i + 1) % 3].getY() + triangle_shift);
        return field;
    }
    
    int[] calcMiddle(Point2D.Double corners[], int triangle_shift) {
        double sum[] = new double[2];
        for(int i = 0; i < 3; i++) {
            sum[0] += corners[i].getX();
            sum[1] += corners[i].getY();
        }
        int mx = (int)(sum[0] /= 3.0) + triangle_shift;
        int my = (int)(sum[1] /= 3.0) + triangle_shift;
        int res[] =  {mx, my};
        return res;
    }
        
   ArrayList<Point> bfs(int field[][], int middle[], int triangle_shift) {
        ArrayList<Point> triangle_points = new ArrayList<Point>();
        int mx = middle[0];
        int my = middle[1];
        field[my][mx] = 0;
        triangle_points.add(new Point(mx - triangle_shift, my - triangle_shift));
        int dir[][] = { {0, 1}, {0, -1}, {1, 0}, {-1, 0} };
        for(int i = 0; i < triangle_points.size(); i++) {
            int curx = triangle_points.get(i).x + triangle_shift;
            int cury = triangle_points.get(i).y + triangle_shift;
            //System.err.println("curx = " + curx + ", cury = " + cury);
            //if(curx == 0 || cury == 0 || curx == triangle_size * 3 - 1 || cury == triangle_size * 3 - 1) continue;
            for(int j = 0; j < 4; j++) {
                int nextx = curx + dir[j][0];
                int nexty = cury + dir[j][1];
                if(field[nexty][nextx] == -1) {
                    field[nexty][nextx] = 0;
                    triangle_points.add(new Point(nextx - triangle_shift, nexty - triangle_shift));
                }
            }
        }
        return triangle_points;
    }

    public void draw(Graphics2D graphics, double iteration) {
        int image_size = Math.min(panel.getHeight(), panel.getWidth()) / 3;
        int image_shift_width = (panel.getWidth() - image_size) / 2;
        int image_shift_height = (panel.getHeight() - image_size) / 2;

        final int triangle_size = image_size / 4;
        final int triangle_shift = 3 * triangle_size / 2;
        int radius = image_size - triangle_size * 2;
        int image_rgb[][] = new int[panel.getWidth()][panel.getHeight()];
        //drawChessboard(image_rgb);
        for(int i = 0; i < 32; i++) {
            double current_angle = iteration * Math.PI * 2 * triangles[i].rotates_count;
            Point2D.Double corners[] = new Point2D.Double[3];
            int field[][] = getTrianglePoints(triangle_size, triangle_shift, i % 2, current_angle, corners);
            ArrayList<Point> triangle_points = bfs(field, calcMiddle(corners, triangle_shift), triangle_shift);

            int dx = (int)(radius * triangles[i].dx);
            int dy = (int)(radius * triangles[i].dy);
            double r = Math.sqrt((double)(dx * dx + dy * dy));
            double start_angle = Math.atan2(-dy, -dx);
            int curdx = dx + (int)(r * Math.cos(iteration * Math.PI * 2+ triangles[i].angle_shift));
            int curdy = dy + (int)(r * Math.sin(iteration * Math.PI * 2+ triangles[i].angle_shift));
            for(Point j : triangle_points) {
                int image_pos_x = (j.x + (int)(image_size * triangles[i].posx));
                int image_pos_y = (j.y + (int)(image_size * triangles[i].posy));
                Point2D.Double real_point = rotate(new Point2D.Double(j.x, j.y), -current_angle);
                real_point.setLocation(real_point.getX() + (int)(image_size * triangles[i].posx),
                        real_point.getY() + (int)(image_size * triangles[i].posy));
                int current_rgb = StaticImageDrawer.getImageRGB(image, (int)real_point.getX() * image.getWidth() / image_size, (int)real_point.getY() * image.getHeight() / image_size);
                Color current_pixel_color = new Color(current_rgb);
                GraphicsPrimitives.putPixel(graphics,
                        image_shift_width + image_pos_x + curdx,
                        image_shift_height + image_pos_y + curdy,
                        current_pixel_color);
            }
            graphics.setColor(Color.black);
            curdx += (int)(image_size * triangles[i].posx);
            curdy += (int)(image_size * triangles[i].posy);
            for(int j = 0; j < 3; j++)
                GraphicsPrimitives.drawLine(graphics,
                        (int)corners[j].getX() + image_shift_width + curdx, (int)corners[j].getY() + image_shift_height + curdy,
                        (int)corners[(j + 1) % 3].getX() + image_shift_width + curdx, (int)corners[(j + 1) % 3].getY() + image_shift_height + curdy);
        }
    }
}
