package labone;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;

/**
 * Created by IntelliJ IDEA.
 * User: zaic
 * Date: 3/20/12
 * Time: 11:21 PM
 */
public class StaticImageDrawer {
    final public static int chess_cell_size = 10;
    final static double alpha_value = 0.7;

    /*
     *  chessboard
     */
    
    public static void fillRectangle(int x, int y, int w, int h, Color color, int res[][]) {
        int color_rgb = color.getRGB();
        for(int i = 0; i < h; i++)
            for(int j = 0; j < w; j++)
                res[y + i][x + i] = color_rgb;
    }
    
    public static void drawChessboard(Graphics2D g, int width, int height) {
        g.setColor(Color.white);
        g.fillRect(0, 0, width, height);
        g.setColor(Color.lightGray);
        boolean start_fill = true;
        for(int i = 0; i < width; i += chess_cell_size) {
            boolean fill = start_fill;
            start_fill = !start_fill;
            for(int j = 0; j < height; j += chess_cell_size) {
                if(fill)
                    g.fillRect(i, j, chess_cell_size, chess_cell_size);
                fill = !fill;
            }
        }
    }

    /*
     *  blend
     */
    
    public static int RGBtoRGBA(int color, int background, double alpha) {
        int red = (int)(((color >> 16) & 0xff) * alpha + ((background >> 16) & 0xff) * (1.0 - alpha));
        int green = (int)(((color >> 8) & 0xff) * alpha + ((background >> 8) & 0xff) * (1.0 - alpha));
        int blue = (int)(((color) & 0xff) * alpha + ((background) & 0xff) * (1.0 - alpha));
        return (red << 16) | (green << 8) | (blue);
    }
    
    /*
     *  filter
     */
    
    public static int getImageRGB(BufferedImage image, int x, int y) {
        if(x < 0) x = 0;
        if(y < 0) y = 0;
        if(x >= image.getWidth()) x = image.getWidth() - 1;
        if(y >= image.getHeight()) y = image.getHeight() - 1;
        return image.getRGB(x, y);
    }
    
    public static int filterColor(int ul, int ur, int dl, int dr, double dx, double dy, int color_shift) {
        ul >>= color_shift;
        ul &= 0xff;
        ur >>= color_shift;
        ur &= 0xff;
        dl >>= color_shift;
        dl &= 0xff;
        dr >>= color_shift;
        dr &= 0xff;
        return (int)(ul * (1.0 - dx) * (1.0 - dy) +
                     ur * dx * (1.0 - dy) +
                     dl * (1.0 - dx) * dy +
                     dr * dx * dy) << color_shift;
    }

    public static int filterImage(BufferedImage image, int x, int y, int image_size) {
        double new_x = 0.5 / image_size + (double)x / image_size;
        double new_y = 0.5 / image_size + (double)y / image_size;
        double old_x = new_x * image.getWidth() + .5;
        double old_y = new_y * image.getHeight() + .5;
        int real_old_x = (int)(old_x) - 1;
        int real_old_y = (int)(old_y) - 1;

        int rgb_upleft = getImageRGB(image, real_old_x, real_old_y);
        int rgb_upright = getImageRGB(image, real_old_x + 1, real_old_y);
        int rgb_downleft = getImageRGB(image, real_old_x, real_old_y + 1);
        int rgb_downright = getImageRGB(image, real_old_x + 1, real_old_y + 1);

        old_x -= (int)old_x;
        old_y -= (int)old_y;
        int r = filterColor(rgb_upleft, rgb_upright, rgb_downleft, rgb_downright, old_x, old_y, 16);
        int g = filterColor(rgb_upleft, rgb_upright, rgb_downleft, rgb_downright, old_x, old_y, 8);
        int b = filterColor(rgb_upleft, rgb_upright, rgb_downleft, rgb_downright, old_x, old_y, 0);
        return r | g | b;
    }

    /*
     *  drawing
     */

    public static void draw(JPanel panel, Graphics2D graphics, BufferedImage image, boolean blend, boolean filter) {
        int image_size = Math.min(panel.getWidth(), panel.getHeight()) / 3;
        int image_shift_width = (panel.getWidth() - image_size) / 2;
        int image_shift_height = (panel.getHeight() - image_size) / 2;
        int image_rgb[][] = new int[image_size][image_size];
        
        if(filter) {
            for(int i = 0; i < image_size; i++)
                for(int j = 0; j < image_size; j++) {
                    image_rgb[i][j] = filterImage(image, i, j, image_size);
                }
        } else {
            for(int i = 0; i < image_size; i++)
                for(int j = 0; j < image_size; j++) {
                    int current_pixel_rgb = image.getRGB(i * image.getWidth() / image_size, j * image.getHeight() / image_size);
                    image_rgb[i][j] = current_pixel_rgb;
                }
        }

        drawChessboard(graphics, panel.getWidth(), panel.getHeight());
        if(blend) {
            for(int i = 0; i < image_size; i++)
                for(int j = 0; j < image_size; j++) {
                    int ci = (i + image_shift_width) % (2 * chess_cell_size) - chess_cell_size;
                    int cj = (j + image_shift_height) % (2 * chess_cell_size) - chess_cell_size;
                    int background_rgb = (ci * cj >= 0 ? Color.lightGray : Color.white).getRGB();
                    image_rgb[i][j] = RGBtoRGBA(image_rgb[i][j], background_rgb, alpha_value);
                }
        }
        
        for(int i = 0; i < image_size; i++) {
            for(int j = 0; j < image_size; j++) {
                Color current_pixel_color = new Color(image_rgb[i][j]);
                GraphicsPrimitives.putPixel(graphics, image_shift_width + i, image_shift_height + j, current_pixel_color);
            }
        }
    }
}
