package labone;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

/**
 * Created by IntelliJ IDEA.
 * User: zaic
 * Date: 3/12/12
 * Time: 10:19 PM
 */
public class Window extends JFrame{
    public static final int iteration_count = 360;

    JTextField txt_filename = null;
    BufferedImage img_lena = null;
    PlotPanel view_lena = null;
    JCheckBox chk_alpha = null;
    JCheckBox chk_animation = null;
    JSlider sld_animation = null;
    Timer animation_timer = null;

    private void addComponentsToPane() {
        // file open dialog
        JPanel file_load_panel = new JPanel();
        file_load_panel.setLayout(new BoxLayout(file_load_panel, BoxLayout.LINE_AXIS));

        JLabel lbl_path = new JLabel("Path:");
        file_load_panel.add(lbl_path);
        file_load_panel.add(Box.createRigidArea(new Dimension(10, 0)));

        txt_filename = new JTextField();
        txt_filename.setEditable(false);
        file_load_panel.add(txt_filename);
        file_load_panel.add(Box.createRigidArea(new Dimension(10, 0)));

        JButton btn_open = new JButton("Open...");
        file_load_panel.add(btn_open);
        ChooseImageFile lst_open = new ChooseImageFile(this);
        btn_open.addActionListener(lst_open);

        // image
        view_lena = new PlotPanel();
        view_lena.setSize(320, 320);
        view_lena.setBorder(new LineBorder(Color.BLUE));

        // bottom
        JPanel bottom_panel = new JPanel();
        bottom_panel.setLayout(new BoxLayout(bottom_panel, BoxLayout.PAGE_AXIS));

        chk_alpha = new JCheckBox();
        chk_alpha.setText("Blend");
        chk_alpha.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                Window.this.view_lena.setBlend(chk_alpha.isSelected());
                Window.this.repaint();
            }
        });
        bottom_panel.add(chk_alpha);

        final JCheckBox chk_filter = new JCheckBox();
        chk_filter.setText("Filter");
        chk_filter.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                Window.this.view_lena.setFilter(chk_filter.isSelected());
                Window.this.repaint();
            }
        });
        bottom_panel.add(chk_filter);

        // animation
        JPanel pnl_animation = new JPanel();
        pnl_animation.setLayout(new BoxLayout(pnl_animation, BoxLayout.LINE_AXIS));
        bottom_panel.add(pnl_animation);
        
        chk_animation = new JCheckBox();
        chk_animation.setText("Animation");
        chk_animation.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(chk_animation.isSelected())
                    initAnimation();
                else
                    dropAnimation();
            }
        });
        pnl_animation.add(chk_animation);
        
        sld_animation = new JSlider(0, iteration_count);
        sld_animation.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                if(chk_animation.isSelected()) {
                    Window.this.view_lena.setAnimation(sld_animation.getValue());
                    Window.this.view_lena.repaint();
                }
            }
        });
        pnl_animation.add(sld_animation);

        JButton btn_init = new JButton("Init");
        btn_init.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                initAnimation();
            }
        });
        pnl_animation.add(btn_init);
        
        JButton btn_start = new JButton("Start");
        btn_start.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if(chk_animation.isSelected())
                    animation_timer.start();
            }
        });
        pnl_animation.add(btn_start);
        
        JButton btn_stop = new JButton("Stop");
        btn_stop.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                animation_timer.stop();
            }
        });
        pnl_animation.add(btn_stop);

        // some code
        Container pane = getContentPane();
        pane.add(file_load_panel, BorderLayout.PAGE_START);
        pane.add(view_lena, BorderLayout.CENTER);
        pane.add(bottom_panel, BorderLayout.PAGE_END);
    }

    Window() {
        animation_timer = new Timer(50, new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                int next_value = sld_animation.getValue() + 1;
                if(next_value > sld_animation.getMaximum())
                    next_value = 0;
                sld_animation.setValue(next_value);
            }
        });
        setTitle("Анимация");
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setPreferredSize(new Dimension(600, 600));

        addComponentsToPane();

        pack();
        setVisible(true);
    }

    public void loadImage(File image_filename) {
        try {
            txt_filename.setText(image_filename.getAbsolutePath());
            BufferedImage img_temp = ImageIO.read(image_filename);
            if(img_temp == null) return ;
            img_lena = img_temp;
        } catch(IOException e) {
            img_lena = null;
            e.printStackTrace();
        }
        dropAnimation();
        view_lena.setImage(img_lena);
        view_lena.repaint();
    }
    
    private void initAnimation() {
        dropAnimation();
        chk_animation.setSelected(true);
        view_lena.setAnimation(0);
        view_lena.repaint();
    }
    
    private void dropAnimation() {
        chk_animation.setSelected(false);
        view_lena.dropAnimation();
        animation_timer.stop();
        sld_animation.setValue(0);
        view_lena.repaint();
    }
}
